@@ -1,34 +1,29 @@
-lavaan2RAM <- function(model, obs.variables = NULL, A.notation="ON", S.notation="WITH",
-                       M.notation="mean", auto.var = TRUE, std.lv = TRUE, ...) {
-    ## if (!requireNamespace("lavaan", quietly=TRUE))    
-    ##     stop("\"lavaan\" package is required for this function.")
-
-    ## Default: fix the latent independent variables at 1
-    my.model <- lavaan::lavaanify(model, fixed.x = FALSE, auto.var=auto.var, std.lv=std.lv, ...)
-    
+## Main function to convert a lavaan parameter table to RAM
+.lavaan2RAM <- function(x, obs.variables = NULL, A.notation="ON", S.notation="WITH",
+                        M.notation="mean") {
     ## replace "." with "" as OpenMx does not like "." in model
-    my.model$plabel <- gsub("\\.", "", my.model$plabel)
+    x$plabel <- gsub("\\.", "", x$plabel)
     ## remove constraints 
     ## there are constraints such as .p1. and .p2; remove them
-    if (length(grep("^\\.", my.model$lhs)) >0 ) my.model <- my.model[-grep("^\\.", my.model$lhs), ]
+    if (length(grep("^\\.", x$lhs)) >0 ) x <- x[-grep("^\\.", x$lhs), ]
     ## label as the variable labels
     #my.model$label <- with(my.model, ifelse(label=="", yes=plabel, no=label))
     
     ## set the starting values in A as 0 if NA
-    if (any((my.model$op=="=~"|my.model$op=="~")&is.na(my.model$ustart))) {
-        my.model[(my.model$op=="=~"|my.model$op=="~")&is.na(my.model$ustart), ]$ustart <- 0
+    if (any((x$op=="=~"|x$op=="~")&is.na(x$ustart))) {
+        x[(x$op=="=~"|x$op=="~")&is.na(x$ustart), ]$ustart <- 0
     }
     
     ## set the starting values in S and free parameters as 0 if NA
-    if (any((my.model$op=="~~"&is.na(my.model$ustart)&my.model$free!=0))) {
-        my.model[my.model$op=="~~"&is.na(my.model$ustart)&my.model$free!=0, ]$ustart <- 0
+    if (any((x$op=="~~"&is.na(x$ustart)&x$free!=0))) {
+        x[x$op=="~~"&is.na(x$ustart)&x$free!=0, ]$ustart <- 0
     }
         
     ## all variables
     ## Removed sort(); otherwise, the variables will be arranged as x1, x10, x2, x3...
-    all.var <- unique(c(my.model$lhs, my.model$rhs))
+    all.var <- unique(c(x$lhs, x$rhs))
     ## latent variables: (with indicators)
-    latent <- unique(my.model[my.model$op== "=~", ]$lhs)
+    latent <- unique(x[x$op== "=~", ]$lhs)
     ## observed variables: not latent
     observed <- all.var[!(all.var %in% latent)]
     ## remove empty string "" when there are mean structure
@@ -59,30 +54,30 @@ lavaan2RAM <- function(model, obs.variables = NULL, A.notation="ON", S.notation=
     Smatrix <- matrix(0, ncol=no.all, nrow=no.all, dimnames=list(all.var, all.var))
     Mmatrix <- matrix(0, nrow=1, ncol=no.all, dimnames=list(1, all.var))
   
-  ## ## latent variable
-  ## lhs <- (all.var %in% my.model[my.model$op== "=~", ]$lhs) 
-  ## ## observed variables
-  ## rhs <- (all.var %in% my.model[my.model$op== "=~", ]$rhs)
+    ## ## latent variable
+    ## lhs <- (all.var %in% my.model[my.model$op== "=~", ]$lhs) 
+    ## ## observed variables
+    ## rhs <- (all.var %in% my.model[my.model$op== "=~", ]$rhs)
 
     ## Prepare the labels
-    for (i in seq_len(nrow(my.model))) {
+    for (i in seq_len(nrow(x))) {
       ## if there is no label
-      if (my.model[i, ]$label=="") {
-        switch(my.model[i, ]$op,
-               "=~" = my.model[i, ]$label <- paste0(my.model[i, ]$rhs, A.notation, my.model[i, ]$lhs),
-               "~"  = my.model[i, ]$label <- paste0(my.model[i, ]$lhs, A.notation, my.model[i, ]$rhs),
-               "~~" = my.model[i, ]$label <- paste0(my.model[i, ]$lhs, S.notation, my.model[i, ]$rhs),
-               "~1" = my.model[i, ]$label <- paste0(my.model[i, ]$lhs, M.notation))
+      if (x[i, ]$label=="") {
+        switch(x[i, ]$op,
+               "=~" = x[i, ]$label <- paste0(x[i, ]$rhs, A.notation, x[i, ]$lhs),
+               "~"  = x[i, ]$label <- paste0(x[i, ]$lhs, A.notation, x[i, ]$rhs),
+               "~~" = x[i, ]$label <- paste0(x[i, ]$lhs, S.notation, x[i, ]$rhs),
+               "~1" = x[i, ]$label <- paste0(x[i, ]$lhs, M.notation))
       }
     }
     
     ## replace NA to 0 in ustart
-    my.model$ustart[is.na(my.model$ustart)] <- 0
+    x$ustart[is.na(x$ustart)] <- 0
     ## keys in as.mxMatrix format
-    key <- with(my.model, ifelse(free==0, yes=ustart, no=paste(ustart, label, sep="*")))  
+    key <- with(x, ifelse(free==0, yes=ustart, no=paste(ustart, label, sep="*")))  
     
-    for (i in seq_len(nrow(my.model))) {
-        my.line <- my.model[i, ]
+    for (i in seq_len(nrow(x))) {
+        my.line <- x[i, ]
         switch(my.line$op,
                ## lhs: IV; rhs: DV
                "=~" = Amatrix[my.line$rhs, my.line$lhs] <- key[i],
@@ -98,3 +93,29 @@ lavaan2RAM <- function(model, obs.variables = NULL, A.notation="ON", S.notation=
 
     list(A=Amatrix, S=Smatrix, F=Fmatrix, M=Mmatrix)
 }
+
+
+lavaan2RAM <- function(model, obs.variables = NULL, A.notation="ON", S.notation="WITH",
+                       M.notation="mean", auto.var = TRUE, std.lv = TRUE,
+                       ngroups=1, ...) {
+    ## if (!requireNamespace("lavaan", quietly=TRUE))    
+    ##     stop("\"lavaan\" package is required for this function.")
+
+    ## Default: fix the latent independent variables at 1
+    my.model <- lavaan::lavaanify(model, fixed.x = FALSE, auto.var=auto.var,
+                                  std.lv=std.lv, ngroups=ngroups, ...)
+
+    ## Drop rows with group==0 
+    my.model <- my.model[!(my.model$group==0), ]
+    
+    if (max(my.model$group)==1) {
+        out <- .lavaan2RAM(my.model, obs.variables=obs.variables, A.notation=A.notation,
+                           S.notation=S.notation, M.notation=M.notation)
+    } else {
+        model.list <- split(my.model, my.model$group)
+        out <- lapply(model.list, .lavaan2RAM,
+                      obs.variables=obs.variables, A.notation=A.notation,
+                      S.notation=S.notation, M.notation=M.notation)
+    }    
+    out
+}
